import ij.*;
import ij.plugin.*;
import ij.plugin.frame.Editor;
import java.io.*;
import java.awt.*;
import java.util.*;
import javax.swing.*;

/** Compiles plugins using the javac compiler. */
public class UroIrmINSTALL_  implements PlugIn{

	private static final String DOSSIER = "UroIrm";


	ImageJ ij ;
	private static sun.tools.javac.Main javac;
	private static ByteArrayOutputStream output;
	public static String separator = System.getProperty("file.separator");
	private static String dir,dir_package,dir_classPath;
	private static final String DOSSIER_PACKAGE = DOSSIER + separator + "UroIrmPackage";
	private static final String DOSSIER_CLASSPATH =  DOSSIER;
	private static final String SAVE_RESULT_FILE = "IJ_PREFDICOM.txt";

	private static Editor errors;
	private String errorsValue ="",msg_err_pb_write,noReadableText,noWritableText,inDir,fileText;
	private static String errorTitle, errorText, errorVersionText;
	private static String installation, installationTitle, installationText ;
	private static String msg_err_default_language_install;
		 
	private static String language = "English";

	private static final String[] name = {
				"Open_Parametre_.java",
				"Open_dicom_stack_.java",
				"UroIrmGraph_.java",
				"UroIrmARoiManager_.java"
			         	};
	private static final String[] packageName = {
			"UroIrmFileDescriptor.java",
			"GraphInterface.java",
			"NullKeyFieldException.java",
			"DicomDictionaryData.java",
			"DicomPref.java",
			"Header.java",
			"DicomHashtable.java",
			"UroIrmDebug.java",
			"UroIrmTime.java",
			"HeaderDicom.java",
			"OptionGraphDialog.java",
			"UroIrmPlot.java",
			"PrefSaveResult.java",
			"UroIrmSaveResult.java",	
			"UroIrmResultDisplay.java"
			};

	public void run (String arg){
		if ( IJ.versionLessThan("1.27"))
			return;
		ij = IJ.getInstance();

		dir = getDir(DOSSIER);
		dir_package=getDir(DOSSIER_PACKAGE);
		dir_classPath= getDir(DOSSIER_CLASSPATH);
		String dir_language = dir_package+separator+ "Langue" ; 
		int total = packageName.length + name.length;
		int count =0;
			
		IJ.showProgress(0);
		if (!showDialogLanguage(dir_language)){
			IJ.showProgress(1);
			return;
		}

		if ( !isReadableFile(dir_package) ){
			IJ.showProgress(1);
			return;
		}


		if ( !setLanguage(dir_language+separator+language)){
			IJ.beep();
			IJ.error(msg_err_default_language_install);
			setLanguage(dir_language+separator+"English");	
		}
	
		for (int i=0; i < packageName.length;i++){
			int index =  packageName[i].indexOf(".java");
			IJ.showStatus(installation + packageName[i].substring(0,index));
			if(  compile(dir_package+separator + packageName[i]) ) 
				IJ.showProgress((double)count/total);
			else{
				showErrors(packageName[i],errorsValue); return;
			}
			count ++;
		}
		for (int i=0; i < name.length;i++){
			int index =  name[i].indexOf(".java");
			IJ.showStatus(installation + name[i].substring(0,index));
			if ( compile(dir+separator +name[i]) )
				IJ.showProgress((double)count/total);
			else {
				showErrors(name[i],errorsValue); return;
			}
			count++;
		}
		IJ.showProgress((double)1);
		IJ.showStatus("done");
		IJ.beep();
		JOptionPane.showMessageDialog(giveNewFrame(),
				installationText,
				installationTitle,
				JOptionPane.INFORMATION_MESSAGE);

	 }
	 
	String getDir(String PREF){
		String prefsDir;
		String homeDir = System.getProperty("user.dir") ;
		String userHome = System.getProperty("user.home");
		String osName = System.getProperty("os.name");
		if (osName.indexOf("Windows",0)>-1)
			prefsDir = homeDir; //ImageJ folder on Windows
		else
			prefsDir = userHome; // Mac Preferences folder or Unix home dir 
		prefsDir +=separator+ "plugins" +separator + PREF;
		return prefsDir;	
	}
	

	boolean isJavac() {
		try {
			if (javac==null) {
				output = new ByteArrayOutputStream(4096);
				javac = new sun.tools.javac.Main(output, "javac");
			}
		} catch (NoClassDefFoundError e) {
			IJ.error("This JVM appears not to include the javac compiler. Javac\n"
				+"is included with the JRE 1.1.8 bundled with the Windows and\n"
 				+"Linux versions of ImageJ. Mac OS 9 users must install Apple's\n"
 				+"Java SDK. When using JDK 1.2 and later, the tools.jar file\n"
 				+"must be added to the command line that runs ImageJ.");
 			return false;
		}
		return true;
	}

	private boolean compile(String path) {
		if (!isJavac())
			return false;
		String classpath = System.getProperty("java.class.path");
		classpath += dir_classPath; 
//IJ.write("classpath: " + classpath);
		output.reset();
		boolean compiled = javac.compile(new String[] {"-deprecation","-classpath",classpath,path});
		errorsValue = output.toString();
		boolean errors = (!compiled || (errorsValue!=null && errorsValue.length()>0));
		if (errors)
			return false;
		return compiled;
	 }

	void showErrors(String name,String s) {
		String text = errorText + name + "\n" ;
		String[] data = {"No details","Details..."};
		IJ.beep();
		int rang = JOptionPane.showOptionDialog(giveNewFrame(),
					text,
					errorTitle,
					JOptionPane.YES_NO_CANCEL_OPTION,
					JOptionPane.ERROR_MESSAGE,
					null,data,data[0]);
		
		if ( rang == 1) {
			if (errors==null || !errors.isVisible())
				errors = (Editor)IJ.runPlugIn("ij.plugin.frame.Editor", "");
			if (errors!=null)
				errors.display("Errors", s);
		}

		IJ.showStatus("done (errors)");
		IJ.showProgress((double)1);
	}

	boolean showDialogLanguage(String path){
		String[] languageData = getLanguage(path);
		String txt = (String)JOptionPane.showInputDialog(giveNewFrame(),
						"Choose your language",
						"Installation program ",	
						JOptionPane.INFORMATION_MESSAGE,
						null,
						languageData,
						languageData[0]);
		if  (txt ==null)
			return false;
		language =txt;	
		if ( txt.equals("Francais") )  {
			errorTitle = " Erreur d'installation";
			errorText = "le programme n'a pas pu install \n la classe : ";
			installation ="Installation de  :  ";
			installationTitle =" Fin de l'installation  ";
			installationText = " L'installation est termine \n " +
						"   Veuillez s'il vous plat, redmarrer ImageJ pour avoir \n" +
						"     la possibilt d'utiliser les programmes \n";
			msg_err_pb_write="Probleme d'ecriture";
			noReadableText = "Veuillez rendre accssible en LECTURE les fichiers suivants : ";
			noWritableText = "Veuillez rendre accssible en ECRITURE les fichiers suivants : ";
			inDir = " * Dans le rpertoire : " ;
			fileText = "   ->  le(s) fichier(s) : ";
			msg_err_default_language_install = " La langue choisie n'a pas pu tre installe. \n "+
						" La language par dfault sera choisie";
						
		} else  {
			errorTitle = " Installation Error";
			errorText = "the program couldn't install \n the class : ";
			installation="Installation  : ";
			installationTitle =" Installation finished ";
			installationText =" The installation is finished  \n "+
						" Please, restart ImageJ in oder to have \n " + 
						" the program avaible \n";
			msg_err_pb_write = "Writting error";
			noReadableText = " Please make sure that the following files are READABLE : ";
			noWritableText = " Please make sure that the following files are WRITABLE : ";
			inDir =" * In the directory : ";
			fileText = "   ->  the file(s) : ";
			msg_err_default_language_install = " The choosen language couldn't be installed.\n"+
							" The default one will be choosen."; 
		}
		return true;
	}

	boolean isReadableFile(String dir){
		boolean status = true;
	// check if all files of Language are readable
		File f = new File(dir + separator + "Langue");
		File[] liste = f.listFiles();
		LinkedList res = new LinkedList();
		int i =0;
		for (i=0;i<liste.length;i++){
			if( !liste[i].canRead() )
				res.add(liste[i].getName());
		}
		status &= (res.size() == 0);
	// check if SAVE_RESULT_FILE exist (if not can be created ) and is readable;
		f = new File(dir+separator+SAVE_RESULT_FILE);
		try {
			if ( !f.exists()) 
				f.createNewFile();
			status &= (f.canRead()&& f.canWrite());  
		} catch (IOException e ){ status = false;}
		if ( status ) return true;

		String text = noReadableText +"\n";
		if ( res.size()!=0 ){
			text += inDir + dir + separator + "Langue \n";
			String aux = fileText ;
			for (ListIterator iter = res.listIterator(0);iter.hasNext();){
				aux += (String)iter.next()+", ";
				text += ( aux.length() >52 )?aux +"\n" : ""; 
				aux = (aux.length() > 52)?"":aux; 
			}
			text += aux + "\n";	
		}
		
		if( !f.canRead())
			text+= inDir+dir+ "\n" + "   " + fileText+SAVE_RESULT_FILE+" \n" ;
		if ( !f.canWrite())
			text+= noWritableText+inDir+dir+ "\n" + "   " + fileText+SAVE_RESULT_FILE+" \n" ;
		text = text.substring(0,text.length()-2); // enlevement de \n
		JOptionPane.showMessageDialog(giveNewFrame(),
							text,
							errorTitle,
							JOptionPane.ERROR_MESSAGE);
		return false;
	}

	String[] getLanguage(String path){
		String[] list = new File(path).list();
		String[] res = new String[list.length]; 
		for (int i=0;i<list.length;i++){
			int index = list[i].indexOf(".txt");
			index = (index < 0 )? list[i].length()-1:index;
			res[i] = list[i].substring(0,index);
		}
		return res;
	}

	boolean setLanguage(String filePath){
		String name = filePath;
		File f = new File(name);
		if ( !f.exists() ){
			name+=".txt";
			f = new File(name);
			if ( !f.exists() )
				return false;
		}
		LinkedList liste = new LinkedList();
		try{
			BufferedReader entry = new BufferedReader(new FileReader(name));
			String line;
			while (true){
				line = entry.readLine();
				if (line == null) 
					break;
				liste.add(line);
			}
			entry.close();

			PrintWriter pw = new PrintWriter(new FileWriter(dir_package+separator+"UroIrmMessage.java"));
			pw.println("package UroIrmPackage;");
			pw.println("");
			pw.println("public interface UroIrmMessage { \n");
			String entete = " public static final String ";
			for (ListIterator e = liste.listIterator(0);e.hasNext();){
				String current = ((String)e.next()).trim();
				if ( current.equals("") || current.startsWith("//"))
					pw.println(current);
				else if ( !current.endsWith(";"))
					pw.println(entete + current+";");
				else 
					pw.println(entete + current);
			}
			pw.println("}");
			pw.close();
		} catch (IOException e) { return false; }

// test whether the new file can compile
		return  compile(dir_package + separator + "UroIrmMessage.java") ;
	}

	Frame giveNewFrame(){
		Frame frame = new Frame();
		if (ij!=null) {
			Image img = ij.getIconImage();
			if (img!=null) frame.setIconImage(img);
		}
		return frame;
	}

}


